/*	CA DSO CA Process Automation extension ArraySearchIndex  */

/*
	Build: 2011-004-07 18:00
	Author: GC
	
	The Field Developed Utilities are provided as part of the Implementation Best Practices. These utilities are written by
	CA staff to aid in deployment of specific CA product releases but are not part of any formal product release. All such
	utilities are reviewed periodically - some become part of a current GA product release and some utilities may be dropped
	or superseded by a future product release. The documentation for specific Field Developed Utilities is updated to reflect
	any such changes. Where Field Utilities have been added to a GA product you must use the GA supported version.
	
	Field Developed Utilities are tested periodically with new releases and significant product maintenance but are not subject
	to formal QA certification. Any change in functionality or support is noted in the documentation, as we become aware of issues.
	You are encouraged to check the documentation for changes before installing new product releases. CA Technical Support will
	assist you in resolving problems on a best effort basis. There is no formal service level for field utilities. If an FDU problem
	you encounter has been corrected in a newer GA release of a CA product, support may direct you to that release.
	
	Field Utility suggestions are handled on a best effort basis by their authors.
	Visit https://support.ca.com/phpdocs/0/common/impcd/r11/ReadMe.htm to review the support general Implementation Best Practices
	policy and learn how to provide feedback for the Implementation Best Practices.
*/

/*	JavaScript function "ArraySearchIndex" that can be instantiated as an object that builds and returns a
 *	array that searched by a key to retrieve indexes in the specified array by with values in the specified
 *	search index field referenced that match the key value.														
 *	
 *	Arguments:
 *	
 *		array
 *
 *		Array of simple objects (i.e., Strings, integers...) or complex objects (i.e., ValueMap objects)
 *
 *		strSearchIndexFieldRef
 *
 *		String reference relative to the array object specified for the field containing values the array should be indexed on when
 *		the array contains complex objects
 *
 *	Usage:
 *
 *	For the example, assuming an array "aryKeyData" of ValueMap objects. The ValueMap have two field named "Key" and "Data".
 *
 *		Index		 Key				  Data
 *		  0			Key 0				Value 0
 *		  1			Key 1				Value 1
 *		  2			Key	2				Value 2
 *		  3			Key 3				Value 3
 *		  4			Key	4				Value 4
 *		  5			Key 5				Value 5
 *
 *	Normally, to find the "Value" where "Key" was equal to "Key 2" if would be necessary to loop through the array, inspect the
 *	value of "Key" and the get the value of "Data":
 *
 *		for (var i = 0; i < aryKeyData.length; i++) {
 *
 *			var key = aryKeyData[i].Key;
 *
 *			if ("Key2" == key) {
 *
 *				var data = aryKeyData[i].Data;
 *
 *				<Perform operations using "data" object returned>
 *				.
 *				.
 *				.
 *			}
 *		}		
 *
 *	To search for the data needed directly, instantiate a new "ArraySearchIndex" object to be indexed on the value of the ValueMap
 *	field "Key": 
 *
 *		var asiDataByKey = new ArraySearchIndex(aryKeyData, "Key");
 *
 *	Instead of looping through all the array elements and checking each ValueMap to determine if the value of the field "Key" matches
 *	the value "Key 1", use the index to return the index of the array element if the key consisted of unique values:
 *
 *		var data = aryKeyData[asiDataByKey["Key 2"]];
 *
 *		<Perform operations using "data" object returned>
 *
 *	NOTE: To verify a unique key was created, check the "isUnque" property
 *
 *		if (asiDataByKey.isUnique) ...
 *
 *	To illustrate handling when the key is not unique, assume the following "aryKeyData" contains the following values:
 *
 *		Index		 Key				  Data
 *		  0			Key 0				Value 0
 *		  1			Key 1				Value 1
 *		  2			Key	2				Value 2
 *		  3			Key 3				Value 3a
 *		  4			Key	3				Value 3b
 *		  5			Key 3				Value 3c
 *
 *
 *	Because the value of the field "Key" not unique multiple array indexes may be return (ex. in the above data a search for
 *	the value "Key 3" should point to array indexes 3, 4 and 5). Below is an example of handling the the search results when
 *	the key values are non-unique:
 *
 *		for (var i in asiDataByKey["Key 1"]) {
 *
 *			var data = aryKeyData[i].Data;
 *
 *			<Perform operations using "data" object returned>
 *			.
 *			.
 *			.
 *		}
 *																																		*/
	ArraySearchIndex = function(array, strSearchIndexFieldRef) {
	
	/*	Initialize key index array	*/
		var aryKeyIndex = new Array();
		
	/*	Append and initialize "isUnique" property for key index array	*/
		aryKeyIndex.isUnique = true;
	
	/*	Begin building search index	*/
		try {
			
			if ("object" == typeof(array)) {
				
				aryKeyIndex = new Array();
				
			/*	Iterate array	*/
				for (var i = 0; i < array.length; i++){
	
				/*	Initialize key	*/
					var key;
	
				/*	Inspect specified search index field reference and take appropriate action */
					if ( (undefined == strSearchIndexFieldRef) || (null == strSearchIndexFieldRef) || ("" == strSearchIndexFieldRef) )
						
					/*	No search index key field was specified: set key to the the current array element value */
						key = array[i];
					else
						
					/*	Search index field was specified: set key to value of referenced field in current array element	*/
						eval("key = array[i]." + strSearchIndexFieldRef);
		
				/*	Inspect key value	*/
					if (undefined == key) {
						 
					/*	Search index field value could not be found: throw exception	*/
						throw {message: "Search index field reference specified not found in array specified."};
					}
					else {
						
					/*	Key is valid	*/
						if (aryKeyIndex[key]){
							
						/*	At least one other element has the same key value: append the current array index to the array of
						 *	array indexes already associated with the key value and set the index "isUnique" property to false	*/
							aryKeyIndex[key][i] = i;
							aryKeyIndex.isUnique = false;					
						}
						else {
						
						/*	No previous instances of the key value have been detected: associate a new array index array with the
						 *	key																										*/
							var aryIndex = new Array();
							aryIndex[i] = i;
							aryKeyIndex[key] = aryIndex;
						}
					}
				}
	
			/*	Return the array of keys and array indexes */
				return aryKeyIndex;
			}
		}
		catch (e){
			
		/*	Set meaning exception message and through exception	*/
			var strExceptionMsg = "ERROR: Exception detected in ArraySearchIndex initialazation. ";
			
			if (-1 < e.message.indexOf("Unexpected end of file")) {
				
				strExceptionMsg = strExceptionMsg + "Search index field reference contains invalid characters.";
			}
			if (-1 < e.message.indexOf("Cannot find default value for object")) {
				
				strExceptionMsg = strExceptionMsg + "Search index field reference specified was undefined, null or empty and array specified does not expose a default field.";
			}
			else {
				
				strExceptionMsg = strExceptionMsg + e.message;
			}
				
			throw {message: strExceptionMsg};
		}	
	};